@@ -1,13 +1,13 @@
-# Copyrights 2004-2007 by Mark Overmeer.
-# For other contributors see ChangeLog.
+# Copyrights 2004,2006-2008 by Mark Overmeer.
+#  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 1.00.
+# Pod stripped from pm file by OODoc 1.03.
 use strict;
 use warnings;
 
 package Math::Polygon::Calc;
 use vars '$VERSION';
-$VERSION = '0.99';
+$VERSION = '1.00';
 use base 'Exporter';
 
 our @EXPORT = qw/
@@ -270,14 +270,20 @@ sub polygon_contains_point($@)
        or croak "ERROR: polygon must be closed: begin==end";
 
     my ($px, $py) = @{ (shift) };
+
     while(@_)
     {   my ($nx, $ny) = @{ (shift) };
-        if(    $py == $ny
-            || ($y <= $py && $y <= $ny)
-            || ($y >  $py && $y >  $ny)
-            || ($x >  $px && $x >  $nx)
+        return 1 if $y==$py && $py==$ny
+                 && ($x >= $px || $x >= $nx)
+                 && ($x <= $px || $x <= $nx);
+
+        if(   $py == $ny
+           || ($y <= $py && $y <= $ny)
+           || ($y >  $py && $y >  $ny)
+           || ($x >  $px && $x >  $nx)
           )
-        {   ($px, $py) = ($nx, $ny);
+        {
+            ($px, $py) = ($nx, $ny);
             next;
         }
 
@@ -292,7 +298,9 @@ sub polygon_contains_point($@)
 
 
 sub polygon_is_closed(@)
-{   my ($first, $last) = @_[0,-1];
+{   @_ or croak "ERROR: empty polygon is neither closed nor open";
+
+    my ($first, $last) = @_[0,-1];
     $first->[0]==$last->[0] && $first->[1]==$last->[1];
 }
 
